Package: Rilostat
Type: Package
Language: en-GB
Title: Access and Manipulation of ILOSTAT Labour Statistics
Version: 2.5.0
Date: 2026-04-24
Authors@R: c(
  person("David", "Bescond", role = c("aut", "cre"),
         email = "bescond@ilo.org",
         comment = c(ORCID = "0000-0002-1642-0304")),
  person("Mabelin", "Villarreal-Fuentes", role = "ctb",
         email = "villarreal-fuentes@ilo.org",
         comment = c(ORCID = "0000-0001-5644-5760")),
  person("International Labour Organization", role = c("cph", "fnd"),
         email = "ilostat@ilo.org")
		)
Maintainer: David Bescond <bescond@ilo.org>
Description: Provides functions to access and manipulate data from the
    ILOSTAT database <https://ilostat.ilo.org>, the International Labour
    Organization's repository of labour statistics. Supports bulk download
    of datasets, metadata retrieval, and tools for filtering, reshaping,
    and analysing data.
License: BSD_2_clause + file LICENSE
Depends: R (>= 4.2.0)
Imports: data.table (>= 1.15.0), dplyr (>= 1.1.4), httr2 (>= 1.0.0),
        haven (>= 2.5.0), rlang (>= 1.1.0), readr (>= 2.1.5), stringr
        (>= 1.5.2), tibble (>= 3.3.0), tidyselect (>= 1.2.0)
Suggests: ggplot2, knitr, plotly, plotrix, rmarkdown, shiny, testthat
        (>= 3.0.0)
URL: https://ilostat.github.io/Rilostat/,
        https://github.com/ilostat/Rilostat
BugReports: https://github.com/ilostat/Rilostat/issues
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-24 15:01:44 UTC; bescond
Author: David Bescond [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1642-0304>),
  Mabelin Villarreal-Fuentes [ctb] (ORCID:
    <https://orcid.org/0000-0001-5644-5760>),
  International Labour Organization [cph, fnd]
Repository: CRAN
Date/Publication: 2026-04-24 15:10:02 UTC
Built: R 4.5.2; ; 2026-04-24 17:23:01 UTC; unix

# Generated by roxygen2: do not edit by hand

export(check_ilostat_version)
export(clean_ilostat_cache)
export(distribution_ilostat)
export(get_ilostat)
export(get_ilostat_dic)
export(get_ilostat_toc)
export(label_ilostat)
export(runapp_ilostat)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,ungroup)
importFrom(haven,read_dta)
importFrom(haven,read_sas)
importFrom(haven,read_sav)
importFrom(haven,write_dta)
importFrom(haven,write_sas)
importFrom(haven,write_sav)
importFrom(httr2,req_headers)
importFrom(httr2,req_perform)
importFrom(httr2,req_timeout)
importFrom(httr2,request)
importFrom(httr2,resp_body_raw)
importFrom(httr2,resp_status)
importFrom(readr,col_character)
importFrom(readr,col_double)
importFrom(readr,cols)
importFrom(readr,read_csv)
importFrom(readr,read_lines)
importFrom(readr,read_rds)
importFrom(rlang,.data)
importFrom(stats,setNames)
importFrom(stringr,fixed)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_replace)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(tibble,as_tibble)
importFrom(tibble,is_tibble)
importFrom(tidyselect,all_of)
importFrom(tidyselect,contains)
importFrom(tidyselect,where)
importFrom(utils,available.packages)
importFrom(utils,download.file)
importFrom(utils,packageVersion)

[![Downloads](http://cranlogs.r-pkg.org/badges/grand-total/Rilostat)](https://cran.r-project.org/package=Rilostat)
[![Downloads](http://cranlogs.r-pkg.org/badges/Rilostat)](https://cran.r-project.org/package=Rilostat)

<br>

## ilostat R package

<!-- README.md is generated from README.Rmd. Please edit that file -->

The ILO’s main online database, [ILOSTAT](https://ilostat.ilo.org),
maintained by the Department of Statistics, is the world’s largest
repository of labour market statistics. It covers all countries and
regions and a wide range of labour-related topics, including employment,
unemployment, wages, working time and labour productivity, to name a
few. It includes time series going back as far as 1938; annual,
quarterly and monthly labour statistics; country-level, regional and
global estimates; and even projections of the main labour market
indicators.

ILOSTAT’s website provides immediate access to all its data and related
metadata through different ways. Basic users can simply view the desired
data online or download it in Excel or csv formats. More advanced users
can take advantage of ILOSTAT’s well-structured [bulk download
facility](https://ilostat.ilo.org/data/bulk/).

The ilostat R package (`'Rilostat'`) was designed to give data users the
ability to access the ILOSTAT database, search for data, rearrange the
information as needed, download it in the desired format, and make
various data visualizations, all in a programmatic and replicable
manner, with the possibility of quickly re-running the queries as
required.

## Main features of the ilostat R package

- Provides access to all annual, quarterly, and monthly data available
  via the ILOSTAT [bulk download
  facility](https://ilostat.ilo.org/data/bulk/)
- Allows to search for and download data and related metadata in
  English, French and Spanish
- Gives the ability to return `POSIXct` dates for easy integration into
  plotting and time-series analysis techniques
- Returns data in long format for direct integration with packages like
  `ggplot2` and `dplyr`
- Gives immediate access to the most recent updates
- Allows for `grep`-style searching for data descriptions and names

## Acknowledgements

The developer of this package drew extensive inspiration from the
[eurostat R package](https://CRAN.R-project.org/package=eurostat) and
its related documentation:\
- [Retrieval and Analysis of Eurostat Open Data with the eurostat
Package](https://journal.r-project.org/articles/RJ-2017-019/RJ-2017-019.pdf) -
[Leo Lahti](https://github.com/antagomir), [Przemyslaw
Biecek](https://github.com/pbiecek), [Markus
Kainu](https://github.com/muuankarski) and [Janne
Huovari](https://github.com/jhuovari). R Journal 9(1), 385-392, 2017.

# Installation

Download and install [R from CRAN](https://cran.r-project.org/), install
[RStudio’s IDE](https://posit.co/products/open-source/rstudio), a
powerful user interface for R. Then use command:

``` r
install.packages("Rilostat")
```

To install the development version, use the following command:

``` r
if(!require(devtools)){install.packages('devtools')}
install_github("ilostat/Rilostat")
```

We do not expect to update the ilostat R package too often, but based on
questions and remarks from ILOSTAT data users, we will progressively
create more examples, tutorials, demos and apps. We will publicly share
all of this via [Rilostat webpage](https://ilostat.github.io/Rilostat/)

## Contribute

Contributions are very welcome:

- [Use issue tracker](https://github.com/ilostat/Rilostat/issues) for
  feedback and bug reports.

- [Star us on the github page](https://github.com/ilostat/Rilostat)

## Permission to reproduce ILO publication and data

The reproduction of ILO material is generally authorized for
non-commercial purposes and within established limits. However, you may
need to submit a formal request in certain circumstances. for more
information please consult:

<https://www.ilo.org/rights-and-permissions>

#' @title Read ilostat Dictionary
#' @description Downloads one ilostat dictionary from ilostat \url{https://ilostat.ilo.org} via bulk download facility 
#' \url{https://ilostat.ilo.org/data/bulk/}.
#' @details For a given coded variable from ilostat \url{https://ilostat.ilo.org/}.
#'    The dictionaries link codes with human-readable labels.
#'    To translate codes to labels, use \code{\link{label_ilostat}}.
#' @param dic A character, dictionary for the variable to be downloaded,
#' @param lang a character, code for language. Available are \code{"en"} (default), 
#'        \code{"fr"} and \code{"es"}. Can be set also with options(ilostat_lang = 'fr'),
#' @param cache_dir a path to a cache directory. The directory has to exist.
#'        	The \code{NULL} (default) uses and creates
#'        	'ilostat' directory in the temporary directory from
#'        	\code{\link{tempdir}}. The directory can also be set with
#'        	\code{option} ilostat_cache_dir,
#' @param quiet a logical, if \code{FALSE} , return message from processing, \code{TRUE} (default).
#' 			Can be set also with options(ilostat_quiet = TRUE), 
#' @return tibble with two columns: code names and full names.
#' @seealso \code{\link{label_ilostat}}, \code{\link{get_ilostat}}.
#' @keywords utilities database
#' @author David Bescond \email{bescond@ilo.org}
#' @references
#' See citation("Rilostat")
#' ilostat bulk download facility user guidelines 
#' \url{https://ilostat.ilo.org/data/bulk/}
#' @examples
#' \dontrun{
#'  tmp <- get_ilostat_dic("indicator")
#'  head(tmp)
#'  tmp <- get_ilostat_dic("classif1", lang = "fr")
#'  head(tmp)
#' }
#' @export
get_ilostat_dic <- function(dic,
                           lang = getOption("ilostat_lang", "en"),
                           cache_dir = getOption("ilostat_cache_dir", NULL),
                           quiet = getOption("ilostat_quiet", TRUE)) {

  dic  <- tolower(dic)
  lang <- tolower(lang)

  dictlang <- paste0(dic, "_", lang)

  # get cache directory
  if (is.null(cache_dir)){
    
	  cache_dir <- file.path(tempdir(), "ilostat")
 	  
  } 
	
	dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)

  # ---- cache file
  
  cache_file <- file.path(cache_dir, paste0(dictlang, ".rds"))



  # ---- READ FROM CACHE ----
  if (file.exists(cache_file)) {

    if (!quiet) { message("Reading dictionary from cache: ", cache_file)}

    get_dic <- readr::read_rds(cache_file)

  } else {

    # ---- DOWNLOAD ----
    url <- paste0(ilostat_url(), "dic/", dictlang, ".rds")

    if (!quiet) {message("Trying dictionary URL '", url, "'")}

    ua <- build_user_agent()

    resp <- request(url) |>
      req_headers(`User-Agent` = ua) |>
      req_timeout(30) |>
      req_perform()

    if (resp_status(resp) != 200) {
      stop("DIC not available '", dictlang, "'")
    }

    tmpfile <- tempfile(fileext = ".rds")
    on.exit(unlink(tmpfile), add = TRUE)

    writeBin(resp_body_raw(resp), tmpfile)

    get_dic <- read_rds(tmpfile) %>%
      dplyr::select(-tidyselect::contains("description"))

    if (!tibble::is_tibble(get_dic)) {
      stop("The dictionary '", dictlang, ".rds' is not a valid tibble")
    }

    # ---- SAVE TO CACHE ----
    readr::write_rds(get_dic, cache_file)

    if (!quiet) { message("Dictionary cached at: ", cache_file) }
  }

  get_dic
}

context("Get")

test_that("get_ilostat includes time and value",{
  skip_on_cran()
  expect_true(all(c("time", "obs_value") %in%
                    names(get_ilostat("UNE_2UNE_SEX_AGE_NB_A"))))
})


test_that("get_ilostat return right classes",{
  skip_on_cran()
  expect_true(all(c("character", "numeric") %in%
                    sapply(get_ilostat("UNE_2UNE_SEX_AGE_NB_A"), class)))

})

context("cache")

test_that("Cache give error if cache dir does not exist", {
  skip_on_cran()
  expect_error(
    get_ilostat("UNE_2UNE_SEX_AGE_NB_A", cache = TRUE, cache_dir = file.path(tempdir(), "test/mytest")))
})

test_that("Cache works", {
  skip_on_cran()
  expect_is({
    t_dir <- file.path(tempdir(), "rilostat")
    dir.create(t_dir)
    k <- get_ilostat("UNE_2UNE_SEX_AGE_NB_A", cache_dir = t_dir)
    },
    "data.frame")
})


context("Search")

test_that("search_ilostat finds",{
  skip_on_cran()
  expect_equal(get_ilostat_toc( search = 
		"KI")$collection[1],
    "KI"
  )
})


context("Label")

test_that("Variable names are labeled",{
  skip_on_cran()
  expect_equal({nrow(get_ilostat('UNE_2UNE_SEX_AGE_NB_A', type = 'label' , filters = list(ref_area.label = 'France'))) > 0}
  
  
  , TRUE)

})

test_that("Dic downloading works", {
  skip_on_cran()
  expect_error(get_ilostat_dic("na_item"))
})

#' @title Switch ilostat codes and labels
#' @description Gets definitions/labels for ilostat codes from ilostat dictionaries.
#' @param x A character or a factor vector or a data_frame to labelled. 
#' @param dic  A string (vector) naming ilostat dictionary or dictionaries.
#'   If \code{NULL} (default) dictionary names are taken from column names of 
#'   the data_frame. A character or a factor vector or a data_frame to labelled, 
#' @param lang a character, code for language. Available are \code{"en"} (default), 
#'        \code{"fr"} and \code{"es"}. Can be set also with options(ilostat_lang = 'fr'),
#' @param code a vector of names of the column for which code columns
#'   should be retained. Set to \code{"all"}, keep all the code.  
#' @details A character or a factor vector of codes returns a corresponding vector of definitions. 
#'   \code{label_ilostat} labels also data_frames from \code{\link{get_ilostat}}. For vectors a dictionary 
#    name have to be supplied. For data_frames dictonary names are taken from column names with suffix ".label". 
#'   "time" and "values" columns are returned as they were, so you can supply data_frame from \code{\link{get_ilostat}} 
#'   and get data_frame with definitions instead of codes.
#' @author David Bescond \email{bescond@ilo.org}
#' @return a vector or a data_frame. The suffix ".label" is added to code column names.
#' @references
#' See citation("Rilostat")
#' ilostat bulk download facility user guidelines 
#' \url{https://ilostat.ilo.org/data/bulk/}
#' @examples
#' \dontrun{
#'  dat <- get_ilostat("UNE_2UNE_SEX_AGE_NB_A", cache = FALSE)
#'  dat_lab <- label_ilostat(dat)
#'  head(dat_lab)
#'
#'
#'  # add just ref_area label
#' require(tidyverse)
#'
#' dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A") %>% 
#'		mutate(ref_area.label = ref_area %>% 
#'		label_ilostat( "ref_area", code = "all"), .after = ref_area)
#'
#'
#'
#'  clean_ilostat_cache() 
#' }
#' @export

label_ilostat <- function(	x, 
							dic = NULL, 
							code = NULL, 
							lang = getOption('ilostat_lang','en')){
  
  if (is_tibble(x)){
    
	y <- x
	
	if(!is.null(code)){
	  
	  if(code[1] %in% 'all'){
	    
		code <- ilostat_cols_ref[ilostat_cols_ref %in% names(y) & !ilostat_cols_ref %in% c('time','obs_value')] 
	  
	  } else {
	  
		code <- code[!stringr::str_detect(code, '.label')]
		}
	
	}

    mynams <- names(y)[!(names(y) %in% c("time", "obs_value")) & names(y) %in% ilostat_cols_ref]
    
	for (i in mynams){
      
	  l <- label_ilostat(y[[i]], dic = i, lang = lang)
      
	  y[[i]] <- as.vector(unlist(l))
	  
	  names(y)[names(y) == i] <- paste0(i, '.label')	
    
	}

    # Codes added if asked
    if (!is.null(code)){ 
	  
	  code_in <- code %in% str_replace(names(y), '.label', '')
	  
	  if (!all(code_in)) {
	    
		stop("code column name(s) ", shQuote(code[!code_in])," not found on x")
	  
	  }
	          
      y_code <- x[, code, drop = FALSE]            
	  
	  y <- as_tibble(cbind(y_code, y))
	  
	  ref_cols <- ilostat_cols_ref[ilostat_cols_ref %in% colnames(y)]
	  
	  order_cols <- c(ref_cols, colnames(y)[!colnames(y) %in% ref_cols])
	  
	  y <- y %>% select(all_of(order_cols))
	
    }
	
  } else {
  
    if (is.null(dic)) stop("Dictionary information is missing")

    dic_df <- get_ilostat_dic(dic, lang = lang)

	if(stringr::str_detect(dic, 'note_')){
	
		# build dictionary lookup once
		dict <- setNames(dic_df[[2]], dic_df[[1]])

		# unique values (equivalent to levels(as.factor(x)))
		ux <- unique(x)

		# compute mapped values
		mapped <- vapply(
		  str_split(ux, "_"),
		  function(z) {
			paste(dict[z], collapse = " | ")
		  },
		  character(1)
		)

		# final mapping
		lookup <- setNames(mapped, ux)

		y <- unname(lookup[x])
			
	} else {
	
      # 
     y <- dic_df[[2]][match(x, dic_df[[1]])]  

	  
    }

  }

  # Ensure the output is tibble

  if (is.data.frame(y) && length(y) > 1) {
    
	y <- as_tibble(y)
  
  }  
  
  
  y

}
 

#' @title R Tools for ilostat Open Data
#'
#' @description This package provides a user-friendly interface to access, download, and manage ILOSTAT open data.
#'
#' @details On a regular basis new tutorials and examples are built and available through this function.
#'
#' \tabular{ll}{
#' Package: \tab Rilostat\cr
#' Type: \tab Package\cr
#' Version: \tab See sessionInfo() or DESCRIPTION file\cr
#' Date: \tab 2020-2025\cr
#' License: \tab BSD_2_clause + file LICENSE\cr
#' LazyLoad: \tab yes\cr
#' }
#'
#' @name Rilostat
#' @author David Bescond \email{bescond@ilo.org}
#' @references
#' See citation("Rilostat")
#'
#' ilostat bulk download facility user guidelines
#' \url{https://ilostat.ilo.org/data/bulk/}
#'
#' @keywords internal
#' @importFrom tidyselect all_of where contains
#' @importFrom dplyr %>% select filter mutate distinct across bind_rows ungroup group_by
#' @importFrom stringr str_replace str_c str_sub str_detect str_split fixed
#' @importFrom readr read_csv col_character col_double cols read_rds read_lines
#' @importFrom haven read_dta read_sav read_sas write_dta write_sav write_sas
#' @importFrom tibble is_tibble as_tibble
#' @importFrom utils download.file packageVersion
#' @importFrom data.table fread fwrite
#' @importFrom stats setNames
#' @importFrom rlang .data
#' @importFrom httr2 request req_headers req_perform resp_body_raw resp_status req_timeout
#' @examples
#' \dontrun{
#' # check which documentation have been recently added:
#' # help(Rilostat)
#'
#' # https://ilostat.github.io/Rilostat/
#' }
"_PACKAGE"


#' Internal utility to determine the base URL for data access.
#'
#' Checks if a local RDS directory exists and returns the local path
#' or the remote API URL.
#'
#' @return A character string representing the base URL or local path.
#' @keywords internal
#' @noRd
ilostat_url <- function() {

  if(dir.exists('/data/rds/')) {
    '/data/rds/'
  } else {
    'https://rplumber.ilo.org/files/'
    }

}

#' Internal utility function to translate and format filters for data processing.
#'
#' Takes user-provided filters (as a list) and converts them into a
#' string suitable for \code{dplyr::filter} evaluation.
#'
#' @param filters A list of filters provided by the user.
#' @param dat_name Column names in the target dataset.
#' @param fixed Logical, whether to use fixed string detection.
#' @return A character string of the filter expression, or \code{NULL}.
#' @keywords internal
#' @noRd
filters_ilostat <- function(
                        filters,
                        dat_name,
                        fixed){

  if(!is.list(filters)){

    if(unique(filters == 'none')){
      filters = list()
    } else{
      # Use base R functions explicitly
      stop("Incorrect filters should be a list or set as 'note'")

    }

  }

  newfilter <- list()

  if (is.list(filters) & !length(filters) == 0){

    names(filters) <- tolower(names(filters)) # not capital sensitive

    
    for (i in 1:length(filters)){

	   if(names(filters[i]) %in% 'timefrom'){

        newfilter[[i]] <- paste0("as.numeric(str_sub(time,1,4)) ", '>= ', as.numeric(filters[[i]]))

      } else  if (names(filters[i]) %in% 'timeto'){

        newfilter[[i]] <- paste0("as.numeric(str_sub(time,1,4)) ", '<= ', as.numeric(filters[[i]]))

      } else {

        if(names(filters[i]) %in% dat_name){
          if(fixed){

          newfilter[[i]] <- paste0(paste0("stringr::str_detect(",  names(filters[i]), ", stringr::fixed('", filters[[i]], "'))"), collapse = ' | ')

        } else {

          newfilter[[i]] <- paste0(paste0("stringr::str_detect(",  names(filters[i]), ", '", filters[[i]], "')"), collapse = ' | ')

        }

        } else {

          message("Variable name: '" ,names(filters[i]), "' , not present on dataset, filters not apply on this variable !")

        }

      }

    }
  }

  if(length(newfilter)>0){

    newfilter <- paste0("(", unlist(newfilter),")", collapse = " & ")

  } else {

    newfilter <- NULL

  }

  newfilter
}

#' Internal environment for session variables.
#'
#' Used to store non-exported, temporary variables needed during a session.
#'
#' @keywords internal
#' @noRd
.ilostatEnv <- new.env()


#' Internal function to run Shiny apps stored remotely.
#'
#' Downloads and executes the GLOBAL.R, UI.R, and SERVER.R files for a given app.
#'
#' @param name The name of the Shiny app directory.
#' @keywords internal
#' @noRd
.runapps <- function(name) {

  production_status <- FALSE

	myGlobal 	<- read_lines(paste0(ilostat_url(), 'apps/',name,'/GLOBAL.r'))
	myUi 		<- read_lines(paste0(ilostat_url(), 'apps/',name,'/UI.r'))
	myServer 	<- read_lines(paste0(ilostat_url(), 'apps/',name,'/SERVER.r'))
 
  # Execute the Shiny app
  shiny::runApp(
    appDir  = list(
      global =  eval(parse(text = myGlobal)),
      ui = function() {
        eval(parse(text = myUi)) 
      },
      server = function(input, output, session) {
         eval(parse(text = myServer)) 
      }
    ),
    port = 3838,
    launch.browser = FALSE,
    host = "127.0.0.1",
    workerId = "",
    quiet = FALSE,
    display.mode = "normal"
  )
}

#' Internal utility to construct a user agent string.
#'
#' Creates a string identifying the package name, version, R platform, and OS.
#'
#' @return A character string for the User-Agent header.
#' @keywords internal
#' @noRd
build_user_agent <- function() {
  sys <- Sys.info()
  os <- paste(sys[["sysname"]], sys[["release"]])
  platform <- R.version$platform

  paste0(
    "Rilostat/", utils::packageVersion("Rilostat"),
    " (", platform, "; ", os, ") "
  )
}

# Set encoding option (not requiring a roxygen block)
options(encoding="UTF-8")

#' @title Play with ILOSTAT data apps
#' @description Open various data exploration and analysis applications hosted by the ILO.
#' @param name `chr` The name of the application to open.
#' 
#'   Available applications:
#'   \itemize{
#'     \item{"dataexplorer"}: Quickly find, filter, pivot, and download ILOSTAT data.
#'     \item{"microquery"}: Explore data availability and create custom queries from the ILO Harmonized Microdata Collection. Users can select variables, filters, and indicators to generate tailored datasets.
#'     \item{"regionalaggregate"}: Generate custom regional estimates by defining or selecting country groupings. The app automatically aggregates country-level modelled data for selected indicators.
#'   }
#' @author David Bescond \email{bescond@ilo.org}
#' @references
#' See citation("Rilostat")
#' 
#' @seealso 
#' Additional documentation for microdata:
#' \itemize{
#'   \item{\href{https://www.ilo.org/publications/ilostat-microdata-processing-quick-guide-principles-and-methods-underlying}{Quick guide on microdata processing}}
#'   \item{\href{https://rplumber.ilo.org/files/website/Ad_Hoc_Micro_ISIC_ISCO_Adjustment_procedure.pdf}{Adjustment procedure for ISCO- and ISIC-based estimates}}
#'   \item{\href{https://rplumber.ilo.org/files/website/Ad_Hoc_Micro_ISIC_ISCO_Global_Regional_Estimates.pdf}{Methodology for ad hoc global and regional estimates}}
#'   \item{\href{https://rplumber.ilo.org/files/website/Ad_Hoc_Micro_Query_Citation.pdf}{Citations}}
#' }
#' 
#' @examples
#' \dontrun{
#'
#'   runapp_ilostat(name = "dataexplorer")
#'
#'   # You can also run other apps like:
#'   runapp_ilostat(name = "microquery")
#'   runapp_ilostat(name = "regionalaggregate")
#'
#'
#' }
#' @export
runapp_ilostat <- function(name = "dataexplorer"){

  # CRITICAL: Check if the 'shiny' package is installed.
  # If 'shiny' is only a suggested package in the DESCRIPTION, it might be missing.
  if (!requireNamespace("shiny", quietly = TRUE)) {
    # If the package is missing, stop execution and give the user clear instructions.
    # 'call. = FALSE' prevents the function call itself from being printed in the error message,
    # making the output cleaner.
    stop(
      "The 'shiny' package is required to run the ILOSTAT web applications.\n",
      "Please install it first: install.packages('shiny')",
      call. = FALSE
    )
  }

  # If 'shiny' is available, proceed to run the application using the internal function.
  .runapps(name)

}

#' @title Clean ilostat Cache
#' @description Deletes all cache files from the your ilostat cache directory.
#'              See \code{\link{get_ilostat}} for more on cache.
#' @param cache_dir A character, path to a cache directory. The directory has to exist.
#'        	The \code{NULL} (default) uses and creates
#'        	'ilostat' directory in the temporary directory from
#'        	\code{\link{tempdir}}. The directory can also be set with
#'        	\code{option} ilostat_cache_dir,
#' @param cache_update a logical whether to delete only out of date cache files. Useful when \code{cache_dir} 
#'          is set as keep only update datasets. Can be set also with
#'        	options(ilostat_update = TRUE). Default is \code{FALSE}.
#' @param quiet a logical, if \code{TRUE} , don't return message from processing, \code{FALSE} (default).
#' 			Can be set also with options(ilostat_quiet = TRUE), 
#' @author David Bescond \email{bescond@ilo.org}
#' @references
#' See citation("Rilostat")
#' ilostat bulk download facility user guidelines 
#' \url{https://ilostat.ilo.org/data/bulk/}
#' @examples 
#' \dontrun{
#' clean_ilostat_cache() 
#' }
#' @export

clean_ilostat_cache <- function(cache_dir = getOption("ilostat_cache_dir", file.path(tempdir(), "ilostat")), 
								cache_update = getOption("ilostat_cache_update", FALSE), 
								quiet = getOption('ilostat_quiet', TRUE)){

  if (!file.exists(cache_dir)){ 
    stop("The cache folder ", cache_dir, " does not exist")
  }
  
  cache_files <- list.files(cache_dir, pattern = "\\.(rds|dta|csv|csv\\.gz|sav|sas7bdat)$", full.names = TRUE)
  
  
  if (length(cache_files) == 0) {
    
	if(!quiet) message("The cache folder ", cache_dir, " is empty.")
  
  } else {
    
	if(cache_update){

      cache_files <-  cache_files %>% as_tibble
  
      cache_files$segment <- stringr::str_split(cache_files$value, '-', n = 2, simplify = TRUE)[,1] 
  
      cache_files$segment <- ifelse(stringr::str_sub(cache_files$segment,-9, -1) %in% 'indicator', 'indicator', 'ref_area')		
   
      cache_files$id <- stringr::str_split(cache_files$value, '-', n = 3, simplify = TRUE)[,2]
  
      cache_files$last.update <- stringr::str_split(cache_files$value, '-', n = 5, simplify = TRUE)[,5]
  
      cache_files$last.update <- stringr::str_split(cache_files$last.update, stringr::fixed('.'), n = 2, simplify = TRUE)[,1]
	
      ref_toc <- NULL

      test = "last.update = ifelse(substr(last.update, 6,8) %in% '/20', last.update %>% strptime('%d/%m/%Y  %H:%M:%S') %>% format('%Y%m%dT%H%M%S'), last.update)"
	
      if ("indicator" %in% cache_files$segment){
            
		ref_toc <- bind_rows(ref_toc, 
						get_ilostat_toc('indicator') %>%
						select(all_of(c("id", "last.update"))) %>%
						mutate(eval(parse(text = test)))
						)
						
		ref_toc <- filter(ref_toc, eval(parse(text = "id %in% cache_files$id")))	
	  
	  }
	  
	 if ("ref_area" %in% cache_files$segment){
	  
	    
		ref_toc <- bind_rows(ref_toc, 
						get_ilostat_toc('ref_area') %>%
						select(all_of(c("id", "last.update"))) %>%
						mutate(eval(parse(text = test))) 
						)
		ref_toc <- filter(ref_toc, eval(parse(text = "id %in% cache_files$id")))				
	  
	  }
	  
	  cache_files <- filter(cache_files, eval(parse(text = "!paste0(id, last.update) %in% paste0(ref_toc$id, ref_toc$last.update)")))
	  
	  if(nrow(cache_files) > 0) {
	  
		cache_files <- cache_files$value
	  
	    unlink(cache_files)
    
	    if(!quiet) message("Deleted outdated ilostat files from ", cache_dir)    
        
	  
	  } else {
	  
		if(!quiet)  message("Cache folder ", cache_dir, " is up to date.")
  
	  
	  }
  
	} else {
  
	
		unlink(cache_files)
    
		if(!quiet) message("Deleted ilostat files from ", cache_dir)    
    
	}
  
  }
  
  invisible(TRUE)

}

#' @title Check Rilostat version against CRAN
#' @description Compare installed version with CRAN and warn if outdated.
#' @param pkg Package name (default "Rilostat")
#' @param quiet Logical, suppress message if up-to-date
#' @return Invisible logical (TRUE = up-to-date, FALSE = outdated or unknown)
#' @importFrom utils packageVersion available.packages
#' @keywords internal
#' @export
check_ilostat_version <- function(pkg = "Rilostat", quiet = FALSE) {
  
  # installed version
  current_version <- tryCatch(
    utils::packageVersion(pkg),
    error = function(e) NULL
  )
  
  if (is.null(current_version)) {
    warning(sprintf("Package '%s' is not installed.", pkg), call. = FALSE)
    return(invisible(FALSE))
  }
  
  # CRAN version (light call)
	cran_version <- tryCatch(
	  as.character(utils::packageVersion(pkg, repos = "https://cloud.r-project.org")),
	  error = function(e) NA
	)
  
  if (is.na(cran_version)) {
    if (!quiet) {      message("Unable to check CRAN version (no internet or CRAN unavailable).")  }
    return(invisible(NA))
  }
  
  # compare
  if (current_version < cran_version) {
    warning(
      sprintf(
        "A newer version of '%s' is available on CRAN (%s > %s). Please run update.packages('%s').",
        pkg, cran_version, current_version, pkg
      ),
      call. = FALSE
    )
    return(invisible(FALSE))
  }
  
  invisible(TRUE)
}


# Internal: run only once per session
.check_version_once <- local({
  checked <- FALSE
  function(...) {
    if (!checked) {
      checked <<- TRUE
      check_ilostat_version(...)
    }
  }
})

#' ilostat_date_convert: Internal Date Conversion Table
#'
#' A static internal tibble used to convert ILOSTAT's time period codes
#' (e.g., 'Q1', 'M03') into partial date strings and numeric fractions
#' for date calculation purposes.
#'
#' @format A tibble with 17 rows and 3 columns:
#' \describe{
#'   \item{code}{ILOSTAT time code (e.g., "Q1", "M07").}
#'   \item{num}{Numeric fraction of the year for calculation (e.g., ".25").}
#'   \item{date}{Partial date string for year-month-day construction (e.g., "-04-01").}
#' }
#' @keywords internal
#' @noRd
ilostat_date_convert <- data.frame(
  code = c("", "Q1", "Q2", "Q3", "Q4", 
           "M01", "M02", "M03", "M04", "M05", "M06", 
           "M07", "M08", "M09", "M10", "M11", "M12"),
  num = c("", "", ".25", ".5", ".75", 
          "", ".083", ".167", ".25", ".333", ".417", 
          ".5", ".583", ".667", ".75", ".833", ".917"),
  date = c("-01-01", "-01-01", "-04-01", "-07-01", "-10-01", 
           "-01-01", "-02-01", "-03-01", "-04-01", "-05-01", "-06-01", 
           "-07-01", "-08-01", "-09-01", "-10-01", "-11-01", "-12-01")
) %>% tibble::as_tibble()

#' ilostat_cols_ref: Standard ILOSTAT Column Names populate
#'
#' A static internal character vector containing the expected default column names
#' returned by the ILOSTAT API. Used for internal data validation and ordering.
#'
#' @format A character vector of length 22.
#' @keywords internal
#' @noRd
ilostat_cols_ref <- c("ref_area", "ref_area.label", "source", "source.label", "indicator", 
                      "indicator.label", "sex", "sex.label", "classif1", "classif1.label", 
                      "classif2", "classif2.label", "time", "obs_value", "obs_status", 
                      "obs_status.label", "note_classif", "note_classif.label", 
                      "note_indicator", "note_indicator.label", "note_source", 
                      "note_source.label")
					  

#' @title Switch ilostat to distribution 
#' @description Get distribution for ilostat number of persons only.
#' @param x  dataset to transform into distribution. 
#' @param var  String variable name use for the distribution default \code{"no"}, 
#' 		could be \code{"sex"}, \code{"classif1"}, \code{"classif2"}.
#' @param .keep if true return only new column call distribution default \code{FALSE},
#' @param quiet a logical, if \code{TRUE} , don't return message from processing, \code{FALSE} (default).
#' 			Can be set also with options(ilostat_quiet = TRUE).
#' @details this function use the max of the corresponding grouping so it is 
#' 		important to not filter any subset of the corresponding variable selected for the distribution
#'		at this level, ie. if you remove SEX_T, the distribution by sex will only have SEX_F or SEX_M / max(SEX_M, SEX_F) * 100,
#'		which is no longer a distribution.
#'
#'		In addition, distribution is only applicable for indicators with Number of persons (usually in thousands), 
#'		So please do not distribute ratios, earnings, hours of works, CPI, GDP etc ... no warning will prevent for that
#'		if doubts use distribution from get_ilostat() instead of, warnings will help you.
#'
#'
#' @author David Bescond \email{bescond@ilo.org}
#' @return a data_frame. obs_status will no longer be a number of persons but a percentage.
#' @references
#' See citation("Rilostat")
#' ilostat bulk download facility user guidelines 
#' \url{https://ilostat.ilo.org/data/bulk/}
#' @examples
#' \dontrun{
#'  dat <- get_ilostat("EMP_TEMP_SEX_STE_GEO_NB_A", cache = FALSE)
#'  dat_dist <- distribution_ilostat(dat, "classif1")
#'  dat_plus_dist <- mutate(dat, dist = distribution_ilostat(dat,"classif1", .keep=TRUE))
#'  head(dat_dist)
#'  clean_ilostat_cache() 
#' }
#' @export

distribution_ilostat <- function(x, var, .keep = FALSE, quiet = getOption('ilostat_quiet', TRUE)) {

  # validation
  if (length(var) != 1 || !tolower(var) %in% c("sex", "classif1", "classif2")) {
    warning("var must be one of: 'sex', 'classif1', 'classif2'. Returning input unchanged.")
    return(x)
  }

  var <- tolower(var)

  if(!quiet) {message(
    paste0(
      "Processing of ",
      paste(unique(x[["indicator"]]), collapse = ", "),
      "... distribution by ", var
    )
  )}

  # reference columns present in data
  ref_col <- ilostat_cols_ref[ilostat_cols_ref %in% names(x)]
  ref_col <- setdiff(ref_col, var)[1]

  # computation
  out <- x %>%
    dplyr::group_by(across(all_of(ref_col))) %>%
	dplyr::mutate(
	  obs_value = dplyr::if_else(
		!is.na(.data$obs_value) & .data$obs_value != 0,
		.data$obs_value / max(.data$obs_value, na.rm = TRUE) * 100,
		.data$obs_value
	  )
	) %>%
    dplyr::ungroup()

  if (.keep) {
    return(out$obs_value)
  }

  out
}

#' @title Read ilostat Table of Contents / workflow
#' @description Download one table of contents from ilostat \url{https://ilostat.ilo.org} via bulk download facility 
#' \url{https://ilostat.ilo.org/data/bulk/}.
#' @param segment A character, way to get datasets by: \code{"indicator"} (default) or \code{"ref_area"}, 
#'        Can be set also with options(ilostat_segment = "ref_area"),
#' @param lang a character, code for language. Available are \code{"en"} (default), 
#'        \code{"fr"} and \code{"es"}. Can be set also with options(ilostat_lang = "fr"),
#' @param search a character vector, "none" (default), datasets with this pattern in
#'  	  the description will be returned,
#' 	      characters vector will be use as AND, Character with "|" as OR, see example,
#'        options(ilostat_time_format = "date"),  
#' @param filters a list; \code{"none"} (default) to get a whole toc or a named list of
#'			filters to get just part of the table. Names of list objects are
#'			ilostat toc variable codes and values are vectors of observation codes.
#'			filters detect on variables.
#' @param fixed a logical, if \code{TRUE} (default), pattern is a string to be matched as is,
#'        Change to \code{FALSE} if more complex regex matching is needed.
#' @param cache_update a logical whether to update cache. Check cache update with last.update attribute store on the cache file name 
#' 			and the one from the table of contents. Can be set also with
#'        	options(ilostat_cache_update = FALSE). Default is \code{TRUE},  
#' @param cache_dir a path to a cache directory. The directory has to exist.
#'        	The \code{NULL} (default) uses and creates
#'        	'ilostat' directory in the temporary directory from
#'        	\code{\link{tempdir}}. The directory can also be set with
#'        	\code{option} ilostat_cache_dir,
#' @param quiet a logical, if \code{FALSE} , return message from processing, \code{TRUE} (default).
#' 			Can be set also with options(ilostat_quiet = TRUE), 
#' @return A tibble with ten columns depending of the segment: indicator or ref_area
#' 		\itemize{
#'      \item{\code{id}} : The codename of dataset of theme, will be used by the get_ilostat and get_ilostat_raw functions,
#'      \item{\code{indicator or ref_area}} : The indicator or ref_area code of dataset,
#'      \item{\code{indicator.label or ref_area.label}} : The indicator or ref_area name of dataset,
#'      \item{\code{freq}}  : The frequency code of dataset,
#'      \item{\code{freq.label}} : Is freq name of dataset,
#'      \item{\code{size}} : Size of the csv.gz files,
#'      \item{\code{data.start}} : First time period of the dataset, 
#'      \item{\code{data.end}} : Last time period of the dataset,
#'      \item{\code{last.update}} : Last update of the dataset,
#'      \item{\code{...}} : Others relevant information
#'      }
#' @seealso \code{\link{get_ilostat}}.
#' @details The toc in English by indicator is downloaded from ilostat API \url{https://rplumber.ilo.org/__docs__/#get-/metadata/toc/indicator/}. 
#' The values in column "id" should be used to download a selected dataset.
#' @details The toc in English by ref_area is downloaded from ilostat API \url{https://rplumber.ilo.org/__docs__/#get-/metadata/toc/ref_area/}. 
#' The values in column "id" should be used to download a selected dataset.
#' @references
#' See citation("Rilostat")
#' ilostat bulk download facility user guidelines \url{https://ilostat.ilo.org/data/bulk/} 
#' @author David Bescond \email{bescond@ilo.org}
#' @keywords utilities database
#' @examples
#' \dontrun{
#' ## default segment by indicator, default lang English
#'  toc <- get_ilostat_toc()
#'  head(toc)
#'  toc <- get_ilostat_toc(segment = "ref_area", lang = "fr")
#'  head(toc)
#' ##
#' ## search on toc
#'  toc <- get_ilostat_toc(search = "education")
#'  head(toc)
#'  toc <- get_ilostat_toc(lang = "fr", search = "age")
#'  head(toc)
#'  toc <- get_ilostat_toc(segment = "ref_area", lang = "fr", search = "Albanie")
#'  toc
#'  toc <- get_ilostat_toc(segment = "ref_area", lang = "es", search = "Trimestral")
#'  head(toc)
#' ##
#' ## search multi on toc
#'  toc <- get_ilostat_toc(segment = "ref_area", lang = "fr", 
#'              search = "Albanie|France", fixed = FALSE)
#'  head(toc)
#'  toc <- get_ilostat_toc(search = "youth|adult", fixed = FALSE)
#'  head(toc)
#' ##
#' }
#' @export

get_ilostat_toc <- function(
  segment = getOption('ilostat_segment', 'indicator'),
  lang = getOption('ilostat_lang', 'en'),
  search = getOption('ilostat_search', 'none'),
  filters = getOption('ilostat_filter', 'none'),
  fixed = getOption('ilostat_fixed', TRUE),
  cache_dir = getOption('ilostat_cache_dir', NULL),
  cache_update = getOption('ilostat_cache_update', TRUE),
  quiet = getOption('ilostat_quiet', TRUE)
) {

  segment <- match.arg(segment, c("indicator", "ref_area"))
  lang <- match.arg(lang, c("en", "fr", "es"))

  # get cache directory
  if (is.null(cache_dir)){
    
	  cache_dir <- file.path(tempdir(), "ilostat")
 	  
  } 
	
	dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)

  # ---- cache file
  
  cache_file <- file.path(
    cache_dir,
    paste0("toc_", segment, "_", lang, ".rds")
  )

  # ---- download if needed
  if (!file.exists(cache_file) || isTRUE(cache_update)) {

    url <- paste0(ilostat_url(), segment, "/table_of_contents_", lang, ".rds")

    if (!quiet) message("Downloading toc from '", url, "'")

    ua <- build_user_agent()

    resp <- request(url) |>
      req_headers(`User-Agent` = ua) |>
      req_timeout(30) |>
      req_perform()

    if (resp_status(resp) != 200) {
      stop("TOC not available for segment '", segment, "' and lang '", lang, "'")
    }

    tmpfile <- tempfile(fileext = ".rds")
    on.exit(unlink(tmpfile), add = TRUE)

    writeBin(resp_body_raw(resp), tmpfile)

    y <- read_rds(tmpfile)

    dir.create(cache_dir, showWarnings = FALSE, recursive = TRUE)
    saveRDS(y, cache_file)

  } else {

    if (!quiet) message("Using cached toc: ", cache_file)
    y <- read_rds(cache_file)
  }

  # ---- safety check
	if (!tibble::is_tibble(y)) {
	  stop("Invalid TOC file: ", cache_file)
	}

# ---- search
if (!identical(search, "none")) {

  y <- y %>%
    mutate(
      titles = do.call(paste0, across(c(2, 3, 4, 5, 11, 12, 13, 14)))
    )

  titles_vec <- y[["titles"]]

  if (fixed) {
    keep <- Reduce(`&`, lapply(search, function(s) {
      stringr::str_detect(titles_vec, stringr::fixed(s))
    }))
  } else {
    keep <- Reduce(`&`, lapply(search, function(s) {
      stringr::str_detect(titles_vec, s)
    }))
  }

  y <- y[keep, ]
  y$titles <- NULL
}

  # ---- filters (kept as-is)
  if (!identical(filters, "none")) {

    filters <- filters_ilostat(filters, names(y), fixed)

    if (!is.null(filters)) {
      y <- filter(y, eval(parse(text = filters)))
    }
  }

  y
}

#' @title Read ilostat Data
#' @description Download datasets from ilostat \url{https://ilostat.ilo.org} via bulk download facility 
#' \url{https://ilostat.ilo.org/data/bulk/}.
#' @param id A code name for the dataset of interest.
#'        See \code{\link{get_ilostat_toc}} or details for how to get code.
#' @param segment A character, way to get datasets by: \code{"indicator"} (default) or \code{"ref_area"}.
#'        Can be set also with options(ilostat_segment = "ref_area"),   
#' @param type a character, type of variables, \code{"code"} (default), \code{"label"} or \code{"both"}.
#'        Can be set also with options(ilostat_type = "both"),  
#' @param lang a character, code for language. Available are \code{"en"} (default), 
#'        \code{"fr"} and \code{"es"}. Can be set also with options(ilostat_lang = "fr"),
#' @param time_format a string giving a type of the conversion of the time
#'        column from the ilostat format. "raw" (default)
#'        does not do conversion and return time as character (ie. "2017", "2017Q1", "2017M01"). A "date" converted to
#'        a \code{\link{Date}} with a first date of the period. A "date_last" converted to a \code{\link{Date}} with
#'        a last date of the period and "num" converted to a numeric. Can be set also with 
#'        options(ilostat_time_format = "date"),  
#' @param filters a list; \code{"none"} (default) to get a whole dataset or a named list of
#'			filters to get just part of the table. Names of list objects are
#'			ilostat variable codes and values are vectors of observation codes.
#'			filters detect on variables, so could be partial, ie. \code{list(sex = "T")} is
#'			enough but equivalent to \code{list(sex = "SEX_T")}. Additional options:
#' 			\itemize{ 
#' 				\item{\code{timefrom}} : starting year of the return dataset,
#' 				\item{\code{timeto}} : ending year of the return dataset.
#'    		}
#' @param best_source a character; \code{"yes"} (default) to get secondary source if exist \code{"all"}, \code{"no"} for secondary only.
#' @param fixed a logical, if \code{TRUE} (default), filters arguments pattern is a string to be matched as is,
#'        Change to \code{FALSE} if more complex regex matching is needed.
#' @param cache a logical whether to do caching. Default is \code{TRUE}. Affects
#' 			only queries from the ilostat bulk download facility. 
#'			Can be set also with options(ilostat_cache = FALSE),
#' @param cache_update a logical whether to update cache. Check cache update with last.update attribute store on the cache file name 
#' 			and the one from the table of contents. Can be set also with
#'        	options(ilostat_cache_update = FALSE). Default is \code{TRUE},  
#' @param cache_dir a path to a cache directory. The directory has to exist.
#'        	The \code{NULL} (default) uses and creates
#'        	'ilostat' directory in the temporary directory from
#'        	\code{\link{tempdir}}. The directory can also be set with
#'        	\code{option} ilostat_cache_dir,
#' @param cache_format a character, format to store on the cache \code{"rds"} (default), but also \code{"csv"}, \code{"dta"}, 
#'        \code{"sav"}, \code{"sas7bdat"}. useful for getting ilostat dataset directly on the \code{cache_dir} without R.  
#' 			Can be set also with options(ilostat_cache_format = "dta"),  
#' @param back a logical; if TRUE (default) return a data frame, otherwise only write to cache,
#' @param cmd a character, R expression use for manipulate internal data frame \code{dat} object applied to each datasets retrieved 
#         after filters and type setting. Manipulation should return data.frame \code{"none"} (default),
#'        If use, \code{cache} is set to FALSE. see examples. 
#'        This argument uses non-standard evaluation and should be used with care.
#' @param quiet a logical, if \code{TRUE} , don't return message from processing, \code{FALSE} (default).
#' 			Can be set also with options(ilostat_quiet = TRUE), 
#' @author David Bescond \email{bescond@ilo.org} 
#'
#' @section others:
#'
#'   Data sets are downloaded from the
#'   ilostat bulk download facility. 
#'   If only the table \code{id} is given, the whole table is downloaded from the
#'   bulk download facility. 
#' 
#'   The bulk download facility is the fastest method to download whole datasets.
#'
#'   By default datasets from the bulk download facility are cached as they are
#'   often rather large. 
#'
#'   Cache files are stored in a temporary directory by default or in
#'   a named directory if cache_dir or option ilostat_cache_dir is defined.
#'   The cache can be emptied with \code{\link{clean_ilostat_cache}}.
#' 
#'   The \code{id}, a code, for the dataset can be searched with
#'   the \code{\link{get_ilostat_toc}} or from the [bulk download facility](https://ilostat.ilo.org/data/bulk/).
#'
#' @references
#' See citation("Rilostat")
#' ilostat bulk download facility user guidelines \url{https://ilostat.ilo.org/data/bulk/}
#' @return a tibble. One column for each dimension in the data and
#'         the values column for numerical values, as well as the metadata columns.
#'         The time column for a time dimension.  
#' @seealso \code{\link{get_ilostat_toc}}, \code{\link{label_ilostat}}
#' @examples 
#' \dontrun{
#' ############# get simple dataset
#'  dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A")
#'  head(dat)
#'  dat <- get_ilostat("NZL_Q", segment = "ref_area")
#'  head(dat)
#'
#'  dir.create(file.path(tempdir(), "r_cache"))
#'  dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A", 
#'                   cache_dir = file.path(tempdir(), "r_cache"))
#'  head(dat)
#'
#'  clean_ilostat_cache(cache_dir = file.path(tempdir(), "r_cache")) 
#'
#'  options(ilostat_update = TRUE)
#'  dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A")
#'  head(dat)
#'  options(ilostat_update = FALSE)
#'  options(ilostat_cache_dir = file.path(tempdir(), "r_cache"))
#'  dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A")
#'
#'  clean_ilostat_cache() 
#'
#' ############# get multiple datasets
#'  dat <- get_ilostat(c("CPI_ACPI_COI_RT_M", "CPI_ACPI_COI_RT_Q"), cache = FALSE)
#'  head(dat)
#'  toc <- get_ilostat_toc(search = "CPI_")
#'  head(toc)
#'  dat <- get_ilostat(toc, cache = FALSE) #id as a tibble
#'
#' ############# get datasets with filters
#'  dat <- get_ilostat(id = c("UNE_TUNE_SEX_AGE_NB_A", "EMP_TEMP_SEX_AGE_NB_A"), 
#'          filters = list(	ref_area = "FRA", 
#'          classif1 = "_YGE15", 
#'          time = "2016",
#'          sex = c("T", "SEX_F")), quiet = TRUE)
#'  head(dat)
#'  clean_ilostat_cache() 
#'
#' ############# store in other format
#'  dir.create(file.path(tempdir(), "ilostat"))
#'
#'  dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A", 
#'                   cache_dir = file.path(tempdir(), "r_cache"), cache_format = "csv")
#'  dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A", 
#'                   cache_dir = file.path(tempdir(), "r_cache"), cache_format = "dta")
#'
#' ############# advanced manipulation
#'
#'  dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A", cmd = "dat %>% count(ref_area)", quiet = TRUE)
#'  label_ilostat(dat, code = "ref_area")
#'
#'  clean_ilostat_cache()
#' }
#' @export

get_ilostat <- function(id, 
						segment = getOption('ilostat_segment', 'indicator'), 
						type = getOption('ilostat_type', 'code'),
						lang = getOption('ilostat_lang', 'en'),
						time_format = getOption('ilostat_time_format', 'raw'),
						filters = getOption('ilostat_filter', 'none'),
						best_source = getOption('ilostat_best_source', 'yes'),
						fixed = getOption('ilostat_fixed', TRUE),
						cache = getOption('ilostat_cache', TRUE),
						cache_update = getOption('ilostat_cache_update', TRUE),
						cache_dir = getOption('ilostat_cache_dir', NULL),
						cache_format = getOption('ilostat_cache_format', 'rds'),
						back = getOption('ilostat_back', TRUE),
						cmd = getOption('ilostat_cmd', 'none'),
						quiet = getOption('ilostat_quiet', TRUE)){

  # get multi id from tibble
  if (is_tibble(id)) { 
    
	ref_id <- unique(id$id) 
  
  } else {
  
    ref_id <- unique(id)
  
  }
 

# normalize best_source into a vector
best_source_vec <- switch(
  tolower(as.character(best_source)),
  "yes" = c(1),
  "no" = c(0),
  "all" = c(1, 0),
  stop("Invalid best_source value. Use 'yes', 'no', or 'all'")
)



  dat <- NULL

 for (i in seq_along(ref_id)) {
	    
    dat <- 	bind_rows(
				
				dat, 
				
				get_ilostat_dat(id = ref_id[i], segment, type, lang, time_format, filters, best_source_vec, fixed, cache, cache_update,	cache_dir, cache_format, back, cmd, quiet 
								
				)
			)
  }

  ref_cols <- ilostat_cols_ref[ilostat_cols_ref %in% colnames(dat)]
	
  order_cols <- c(ref_cols, colnames(dat)[!colnames(dat) %in% ref_cols])
	
  dat %>% select(all_of(order_cols))
 
}


get_ilostat_dat <- function(id, 
							segment, 
							type, 
							lang, 
							time_format,
							filters,
							best_source_vec,
							fixed,
							cache, 
							cache_update, 
							cache_dir,
							cache_format,
							back,
							cmd, 
							quiet 
							){

  # check id validity and return last update
  ref_id <- id
  last_toc_update <- get_ilostat_toc(segment, lang) %>% 
					 filter(id %in% ref_id) 
  last_toc_update <- unique(last_toc_update$last.update)
					
  last_toc_update <- ifelse(substr(last_toc_update, 6,8) %in% '/20', 
												format(strptime(last_toc_update, '%d/%m/%Y  %H:%M'), '%Y%m%dT%H%M'), 
												last_toc_update) 
					
  if(length(last_toc_update) == 0){
    
	if(!quiet){message("Dataset with id = '", id, "' does not exist or is not readable")}
	
	return(NULL)
  
  }
  
  # get cache directory
  if (is.null(cache_dir)){
    
	  cache_dir <- file.path(tempdir(), "ilostat")
 	  
  } 
	
	dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
	
  # cache filename
  cache_file <- file.path(cache_dir, paste0(segment, "-", id, "-", type,"-",time_format, "-", last_toc_update, "-", paste0(best_source_vec, collapse = "") , paste0(".", cache_format)))
  
  if (cache){
    
	if(cache_update & file.exists(cache_file)){
	  
	  cache_update <- FALSE
	
	  if(!quiet){message("Table ", id, " is up to date")}
  
    }  
  
  }
  
  # if cache = FALSE or update or new: download else read from cache
  if (!cache || cache_update || !file.exists(cache_file) ){
      
	  dat <- get_ilostat_raw(id, segment, cache_file, cache_dir, cache_format, quiet)   %>% filter(.data$best_source %in% best_source_vec)	
		
	
	  

		# delete alod cache 
	  delete_old_cache(cache_dir, cache_file, quiet)
  
	  
      if (type == 'code') {
        
		dat <- dat 
	  
	  } else if(type == 'both') {
		
		dat <- label_ilostat(dat, code = 'all', lang = lang)
	  
	  } else if (type == 'label') {
        
		dat <- label_ilostat(dat, lang = lang)
      
	  } else {
        
		stop("Invalid type.")
      
	  }
  
  } else {
	
    cf <- path.expand(cache_file)
 
	if(cache_format == 'rds'){
		
		dat <- read_rds(cache_file) %>% as_tibble %>% mutate(across(where(is.factor), as.character))
	
	}
	
	if(cache_format == 'csv'){
		
		dat <- read_csv(cache_file, col_types = cols(.default = col_character(), obs_value = col_double()), progress = FALSE)
	
	}
	
	if(cache_format == 'dta'){
		
		dat <- read_dta(cache_file)
		
		colnames(dat) <- colnames(dat) %>% stringr::str_replace(stringr::fixed('label'), stringr::fixed('.label'))
		
	} 
	
	if(cache_format == 'sav'){
		
		dat <- read_sav(cache_file)
	
	} 
	
	if(cache_format == 'sas7bdat'){
		
		dat <- read_sas(cache_file)
		
		colnames(dat) <- colnames(dat) %>% stringr::str_replace(stringr::fixed('label'), stringr::fixed('.label'))
	
	} 
	
	if(!quiet){message("Table ", id, " read from cache file: ", cf)}
  
  }
  


# process time_format
if (tolower(time_format) %in% "num" && str_sub(id, -1, -1) %in% c("Q", "M")) {

  # build lookup once
  lookup <- setNames(ilostat_date_convert[["num"]], ilostat_date_convert$code)

  dat$time <- paste0(
    str_sub(dat$time, 1, 4),
    lookup[str_sub(dat$time, 5, -1)]
  ) |> as.numeric()

} else if (tolower(time_format) %in% "num") {

  dat$time <- as.numeric(dat$time)

}
 

if (tolower(str_sub(time_format, 1, 4)) %in% "date") {

  if (str_sub(id, -1, -1) %in% c("Q", "M")) {

    # lookup for code -> date
    lookup_date <- setNames(ilostat_date_convert[["date"]], ilostat_date_convert$code)

    dat$time <- paste0(
      str_sub(dat$time, 1, 4),
      lookup_date[str_sub(dat$time, 5, -1)]
    )

  } else {

    dat$time <- paste0(dat$time, "-01-01")

  }

  if (tolower(time_format) %in% "date_last") {

    if (str_sub(id, -1, -1) %in% c("Q", "M")) {

      shift <- c("A" = 367, "Q" = 96, "M" = 32)[str_sub(id, -1, -1)]

      # unique dates (equivalent to levels(as.factor()))
      u <- unique(dat$time)

      # build lookup: original -> shifted end-of-period
      lookup_last <- setNames(
        as.Date(cut(as.Date(u) + shift, "month")) - 1,
        u
      )

      dat$time <- lookup_last[dat$time]

    } else {

      dat$time <- stringr::str_replace(dat$time, "-01-01", "-12-31")

    }
  }



}
  
  
  
  # if save in cache_dir, cache_format
  if (cache && (cache_update || !file.exists(cache_file))){

    # resort colnames

	ref_cols <- ilostat_cols_ref[ilostat_cols_ref %in% colnames(dat)]
	
	order_cols <- c(ref_cols, colnames(dat)[!colnames(dat) %in% ref_cols])
	
	dat <- dat %>% select(all_of(order_cols))
	
	

	if(cache_format == 'rds'){
		
		saveRDS(dat, file = cache_file)
	
	}
	
	if(cache_format == 'csv'){
		
		fwrite(dat, file = cache_file)
	
	}
	
	if(cache_format == 'dta'){
		
		colnames(dat) <- colnames(dat) %>% stringr::str_replace(stringr::fixed('.label'), stringr::fixed('label'))
		
		write_dta(dat, path = cache_file)
		
		colnames(dat) <- colnames(dat) %>% stringr::str_replace(stringr::fixed('label'), stringr::fixed('.label'))
		
	}
	
	if(cache_format == 'sav'){
		
		write_sav(dat, path = cache_file)
	
	}
	
	if(cache_format == 'sas7bdat'){
		
		colnames(dat) <- colnames(dat) %>% stringr::str_replace(stringr::fixed('.label'), stringr::fixed('label'))
		
		write_sas(dat, path = cache_file)
		
		colnames(dat) <- colnames(dat) %>% stringr::str_replace(stringr::fixed('label'), stringr::fixed('.label'))
	
	}
	
	if(!quiet){ message("Table ", id, " cached at ", path.expand(cache_file))}
	
  }
  
  if(!back){return(NULL)}

  # process filters
  if (!identical(filters, "none")){ 
    
	filters <- filters_ilostat(filters, names(dat), fixed)
	  
    if(!is.null(filters)){
        
		dat <- dat %>% filter(!!rlang::parse_expr(filters))
	  
    }

  }	
  
  
  
 
  

  # process cmd  
  if(cmd != 'none'){
	
	try(dat <- rlang::eval_tidy(rlang::parse_expr(cmd), data = list(dat = dat)), silent = TRUE)
	

  }	
  
	if(length(best_source_vec) == 1 && best_source_vec == 1){
		dat %>% select(-contains("best_source"))} else {
		dat
		}		
    	   
}



get_ilostat_raw <- function(id, 
                            segment,   
                            cache_file, 
                            cache_dir, 
                            cache_format, 
                            quiet) {

  base <- paste0(ilostat_url(), segment, "/", id, ".rds")	   



  if (str_sub(ilostat_url(), 1, 5) == "https") {
    tfile <- cache_file %>%
      stringr::str_replace(paste0(stringr::fixed('.'), cache_format), ".rds")

    url <- paste0("https://rplumber.ilo.org/files/", segment, "/", id, ".rds")
    if(!quiet){message("Trying URL '", url, "'")}

    # Build User-Agent with platform + OS
    ua <- build_user_agent()

	# need to add : `Accept-Encoding`: br, gzip

    # Perform request with User-Agent
    resp <- request(url) |>
	  req_headers(`User-Agent` = ua) |>
      httr2::req_timeout(40) |>  # wait up to 40 seconds
      req_perform()

    dat <- NULL
    if (resp_status(resp) == 200) {
      writeBin(resp_body_raw(resp), tfile)
      dat <- read_rds(tfile) %>%
        as_tibble() %>%
        mutate(across(where(is.factor), as.character))

      if (!quiet) message("Cache saved at '", tfile, "'")
    }

  } else {
    dat <- tryCatch(
      read_rds(base) %>%
        as_tibble() %>%
        mutate(across(where(is.factor), as.character)),
      error = function(e) NULL
    )
  }

  # check validity
  if (!is_tibble(dat)) {
    stop("Dataset with id = '", id, "' does not exist or is not readable")
  }

	dat
  
}



delete_old_cache <- function(cache_dir, new_file_path, quiet) {
  
  # Step 1: Extract the segment-id from the new file's name.
  filename <- basename(new_file_path)
  segment_id_pattern <- sub("([[:alnum:]]+-[[:alnum:]]+)-.*", "\\1", filename)
  
  # Step 2: List all files in the cache directory that match the segment-id pattern.
  matching_files <- list.files(
    path = cache_dir,
    pattern = paste0("^", segment_id_pattern, "-"),
    full.names = TRUE
  )
  
  # Step 3: Exclude the new file from the list of files to be deleted.
  files_to_delete <- setdiff(matching_files, new_file_path)

  # Step 4: Delete the matched files.
  if (length(files_to_delete) > 0) {
    if(! quiet) message("Deleting the following old cache files:\n", paste(files_to_delete, collapse = "\n"))
    file.remove(files_to_delete)
  } else {
    if(! quiet) message("No old cache files found to delete for this segment-id.")
  }

  invisible(files_to_delete)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ilostat.R
\name{get_ilostat}
\alias{get_ilostat}
\title{Read ilostat Data}
\usage{
get_ilostat(
  id,
  segment = getOption("ilostat_segment", "indicator"),
  type = getOption("ilostat_type", "code"),
  lang = getOption("ilostat_lang", "en"),
  time_format = getOption("ilostat_time_format", "raw"),
  filters = getOption("ilostat_filter", "none"),
  best_source = getOption("ilostat_best_source", "yes"),
  fixed = getOption("ilostat_fixed", TRUE),
  cache = getOption("ilostat_cache", TRUE),
  cache_update = getOption("ilostat_cache_update", TRUE),
  cache_dir = getOption("ilostat_cache_dir", NULL),
  cache_format = getOption("ilostat_cache_format", "rds"),
  back = getOption("ilostat_back", TRUE),
  cmd = getOption("ilostat_cmd", "none"),
  quiet = getOption("ilostat_quiet", TRUE)
)
}
\arguments{
\item{id}{A code name for the dataset of interest.
See \code{\link{get_ilostat_toc}} or details for how to get code.}

\item{segment}{A character, way to get datasets by: \code{"indicator"} (default) or \code{"ref_area"}.
Can be set also with options(ilostat_segment = "ref_area"),}

\item{type}{a character, type of variables, \code{"code"} (default), \code{"label"} or \code{"both"}.
Can be set also with options(ilostat_type = "both"),}

\item{lang}{a character, code for language. Available are \code{"en"} (default), 
\code{"fr"} and \code{"es"}. Can be set also with options(ilostat_lang = "fr"),}

\item{time_format}{a string giving a type of the conversion of the time
column from the ilostat format. "raw" (default)
does not do conversion and return time as character (ie. "2017", "2017Q1", "2017M01"). A "date" converted to
a \code{\link{Date}} with a first date of the period. A "date_last" converted to a \code{\link{Date}} with
a last date of the period and "num" converted to a numeric. Can be set also with 
options(ilostat_time_format = "date"),}

\item{filters}{a list; \code{"none"} (default) to get a whole dataset or a named list of
filters to get just part of the table. Names of list objects are
ilostat variable codes and values are vectors of observation codes.
filters detect on variables, so could be partial, ie. \code{list(sex = "T")} is
enough but equivalent to \code{list(sex = "SEX_T")}. Additional options:
    \itemize{ 
        \item{\code{timefrom}} : starting year of the return dataset,
        \item{\code{timeto}} : ending year of the return dataset.
   }}

\item{best_source}{a character; \code{"yes"} (default) to get secondary source if exist \code{"all"}, \code{"no"} for secondary only.}

\item{fixed}{a logical, if \code{TRUE} (default), filters arguments pattern is a string to be matched as is,
Change to \code{FALSE} if more complex regex matching is needed.}

\item{cache}{a logical whether to do caching. Default is \code{TRUE}. Affects
    only queries from the ilostat bulk download facility. 
Can be set also with options(ilostat_cache = FALSE),}

\item{cache_update}{a logical whether to update cache. Check cache update with last.update attribute store on the cache file name 
 and the one from the table of contents. Can be set also with
options(ilostat_cache_update = FALSE). Default is \code{TRUE},}

\item{cache_dir}{a path to a cache directory. The directory has to exist.
The \code{NULL} (default) uses and creates
'ilostat' directory in the temporary directory from
\code{\link{tempdir}}. The directory can also be set with
\code{option} ilostat_cache_dir,}

\item{cache_format}{a character, format to store on the cache \code{"rds"} (default), but also \code{"csv"}, \code{"dta"}, 
\code{"sav"}, \code{"sas7bdat"}. useful for getting ilostat dataset directly on the \code{cache_dir} without R.  
     Can be set also with options(ilostat_cache_format = "dta"),}

\item{back}{a logical; if TRUE (default) return a data frame, otherwise only write to cache,}

\item{cmd}{a character, R expression use for manipulate internal data frame \code{dat} object applied to each datasets retrieved 
If use, \code{cache} is set to FALSE. see examples. 
This argument uses non-standard evaluation and should be used with care.}

\item{quiet}{a logical, if \code{TRUE} , don't return message from processing, \code{FALSE} (default).
Can be set also with options(ilostat_quiet = TRUE),}
}
\value{
a tibble. One column for each dimension in the data and
        the values column for numerical values, as well as the metadata columns.
        The time column for a time dimension.
}
\description{
Download datasets from ilostat \url{https://ilostat.ilo.org} via bulk download facility 
\url{https://ilostat.ilo.org/data/bulk/}.
}
\section{others}{


  Data sets are downloaded from the
  ilostat bulk download facility. 
  If only the table \code{id} is given, the whole table is downloaded from the
  bulk download facility. 

  The bulk download facility is the fastest method to download whole datasets.

  By default datasets from the bulk download facility are cached as they are
  often rather large. 

  Cache files are stored in a temporary directory by default or in
  a named directory if cache_dir or option ilostat_cache_dir is defined.
  The cache can be emptied with \code{\link{clean_ilostat_cache}}.

  The \code{id}, a code, for the dataset can be searched with
  the \code{\link{get_ilostat_toc}} or from the [bulk download facility](https://ilostat.ilo.org/data/bulk/).
}

\examples{
\dontrun{
############# get simple dataset
 dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A")
 head(dat)
 dat <- get_ilostat("NZL_Q", segment = "ref_area")
 head(dat)

 dir.create(file.path(tempdir(), "r_cache"))
 dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A", 
                  cache_dir = file.path(tempdir(), "r_cache"))
 head(dat)

 clean_ilostat_cache(cache_dir = file.path(tempdir(), "r_cache")) 

 options(ilostat_update = TRUE)
 dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A")
 head(dat)
 options(ilostat_update = FALSE)
 options(ilostat_cache_dir = file.path(tempdir(), "r_cache"))
 dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A")

 clean_ilostat_cache() 

############# get multiple datasets
 dat <- get_ilostat(c("CPI_ACPI_COI_RT_M", "CPI_ACPI_COI_RT_Q"), cache = FALSE)
 head(dat)
 toc <- get_ilostat_toc(search = "CPI_")
 head(toc)
 dat <- get_ilostat(toc, cache = FALSE) #id as a tibble

############# get datasets with filters
 dat <- get_ilostat(id = c("UNE_TUNE_SEX_AGE_NB_A", "EMP_TEMP_SEX_AGE_NB_A"), 
         filters = list(	ref_area = "FRA", 
         classif1 = "_YGE15", 
         time = "2016",
         sex = c("T", "SEX_F")), quiet = TRUE)
 head(dat)
 clean_ilostat_cache() 

############# store in other format
 dir.create(file.path(tempdir(), "ilostat"))

 dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A", 
                  cache_dir = file.path(tempdir(), "r_cache"), cache_format = "csv")
 dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A", 
                  cache_dir = file.path(tempdir(), "r_cache"), cache_format = "dta")

############# advanced manipulation

 dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A", cmd = "dat \%>\% count(ref_area)", quiet = TRUE)
 label_ilostat(dat, code = "ref_area")

 clean_ilostat_cache()
}
}
\references{
See citation("Rilostat")
ilostat bulk download facility user guidelines \url{https://ilostat.ilo.org/data/bulk/}
}
\seealso{
\code{\link{get_ilostat_toc}}, \code{\link{label_ilostat}}
}
\author{
David Bescond \email{bescond@ilo.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_ilostat.R
\name{label_ilostat}
\alias{label_ilostat}
\title{Switch ilostat codes and labels}
\usage{
label_ilostat(
  x,
  dic = NULL,
  code = NULL,
  lang = getOption("ilostat_lang", "en")
)
}
\arguments{
\item{x}{A character or a factor vector or a data_frame to labelled.}

\item{dic}{A string (vector) naming ilostat dictionary or dictionaries.
If \code{NULL} (default) dictionary names are taken from column names of 
the data_frame. A character or a factor vector or a data_frame to labelled,}

\item{code}{a vector of names of the column for which code columns
should be retained. Set to \code{"all"}, keep all the code.}

\item{lang}{a character, code for language. Available are \code{"en"} (default), 
\code{"fr"} and \code{"es"}. Can be set also with options(ilostat_lang = 'fr'),}
}
\value{
a vector or a data_frame. The suffix ".label" is added to code column names.
}
\description{
Gets definitions/labels for ilostat codes from ilostat dictionaries.
}
\details{
A character or a factor vector of codes returns a corresponding vector of definitions. 
  \code{label_ilostat} labels also data_frames from \code{\link{get_ilostat}}. For vectors a dictionary 
  "time" and "values" columns are returned as they were, so you can supply data_frame from \code{\link{get_ilostat}} 
  and get data_frame with definitions instead of codes.
}
\examples{
\dontrun{
 dat <- get_ilostat("UNE_2UNE_SEX_AGE_NB_A", cache = FALSE)
 dat_lab <- label_ilostat(dat)
 head(dat_lab)


 # add just ref_area label
require(tidyverse)

dat <- get_ilostat("UNE_TUNE_SEX_AGE_NB_A") \%>\% 
	mutate(ref_area.label = ref_area \%>\% 
	label_ilostat( "ref_area", code = "all"), .after = ref_area)



 clean_ilostat_cache() 
}
}
\references{
See citation("Rilostat")
ilostat bulk download facility user guidelines 
\url{https://ilostat.ilo.org/data/bulk/}
}
\author{
David Bescond \email{bescond@ilo.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_version.R
\name{check_ilostat_version}
\alias{check_ilostat_version}
\title{Check Rilostat version against CRAN}
\usage{
check_ilostat_version(pkg = "Rilostat", quiet = FALSE)
}
\arguments{
\item{pkg}{Package name (default "Rilostat")}

\item{quiet}{Logical, suppress message if up-to-date}
}
\value{
Invisible logical (TRUE = up-to-date, FALSE = outdated or unknown)
}
\description{
Compare installed version with CRAN and warn if outdated.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runapp_ilostat.R
\name{runapp_ilostat}
\alias{runapp_ilostat}
\title{Play with ILOSTAT data apps}
\usage{
runapp_ilostat(name = "dataexplorer")
}
\arguments{
\item{name}{`chr` The name of the application to open.

  Available applications:
  \itemize{
    \item{"dataexplorer"}: Quickly find, filter, pivot, and download ILOSTAT data.
    \item{"microquery"}: Explore data availability and create custom queries from the ILO Harmonized Microdata Collection. Users can select variables, filters, and indicators to generate tailored datasets.
    \item{"regionalaggregate"}: Generate custom regional estimates by defining or selecting country groupings. The app automatically aggregates country-level modelled data for selected indicators.
  }}
}
\description{
Open various data exploration and analysis applications hosted by the ILO.
}
\examples{
\dontrun{

  runapp_ilostat(name = "dataexplorer")

  # You can also run other apps like:
  runapp_ilostat(name = "microquery")
  runapp_ilostat(name = "regionalaggregate")


}
}
\references{
See citation("Rilostat")
}
\seealso{
Additional documentation for microdata:
\itemize{
  \item{\href{https://www.ilo.org/publications/ilostat-microdata-processing-quick-guide-principles-and-methods-underlying}{Quick guide on microdata processing}}
  \item{\href{https://rplumber.ilo.org/files/website/Ad_Hoc_Micro_ISIC_ISCO_Adjustment_procedure.pdf}{Adjustment procedure for ISCO- and ISIC-based estimates}}
  \item{\href{https://rplumber.ilo.org/files/website/Ad_Hoc_Micro_ISIC_ISCO_Global_Regional_Estimates.pdf}{Methodology for ad hoc global and regional estimates}}
  \item{\href{https://rplumber.ilo.org/files/website/Ad_Hoc_Micro_Query_Citation.pdf}{Citations}}
}
}
\author{
David Bescond \email{bescond@ilo.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ilostat_dic.R
\name{get_ilostat_dic}
\alias{get_ilostat_dic}
\title{Read ilostat Dictionary}
\usage{
get_ilostat_dic(
  dic,
  lang = getOption("ilostat_lang", "en"),
  cache_dir = getOption("ilostat_cache_dir", NULL),
  quiet = getOption("ilostat_quiet", TRUE)
)
}
\arguments{
\item{dic}{A character, dictionary for the variable to be downloaded,}

\item{lang}{a character, code for language. Available are \code{"en"} (default), 
\code{"fr"} and \code{"es"}. Can be set also with options(ilostat_lang = 'fr'),}

\item{cache_dir}{a path to a cache directory. The directory has to exist.
The \code{NULL} (default) uses and creates
'ilostat' directory in the temporary directory from
\code{\link{tempdir}}. The directory can also be set with
\code{option} ilostat_cache_dir,}

\item{quiet}{a logical, if \code{FALSE} , return message from processing, \code{TRUE} (default).
Can be set also with options(ilostat_quiet = TRUE),}
}
\value{
tibble with two columns: code names and full names.
}
\description{
Downloads one ilostat dictionary from ilostat \url{https://ilostat.ilo.org} via bulk download facility 
\url{https://ilostat.ilo.org/data/bulk/}.
}
\details{
For a given coded variable from ilostat \url{https://ilostat.ilo.org/}.
   The dictionaries link codes with human-readable labels.
   To translate codes to labels, use \code{\link{label_ilostat}}.
}
\examples{
\dontrun{
 tmp <- get_ilostat_dic("indicator")
 head(tmp)
 tmp <- get_ilostat_dic("classif1", lang = "fr")
 head(tmp)
}
}
\references{
See citation("Rilostat")
ilostat bulk download facility user guidelines 
\url{https://ilostat.ilo.org/data/bulk/}
}
\seealso{
\code{\link{label_ilostat}}, \code{\link{get_ilostat}}.
}
\author{
David Bescond \email{bescond@ilo.org}
}
\keyword{database}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution_ilostat.r
\name{distribution_ilostat}
\alias{distribution_ilostat}
\title{Switch ilostat to distribution}
\usage{
distribution_ilostat(
  x,
  var,
  .keep = FALSE,
  quiet = getOption("ilostat_quiet", TRUE)
)
}
\arguments{
\item{x}{dataset to transform into distribution.}

\item{var}{String variable name use for the distribution default \code{"no"}, 
could be \code{"sex"}, \code{"classif1"}, \code{"classif2"}.}

\item{.keep}{if true return only new column call distribution default \code{FALSE},}

\item{quiet}{a logical, if \code{TRUE} , don't return message from processing, \code{FALSE} (default).
Can be set also with options(ilostat_quiet = TRUE).}
}
\value{
a data_frame. obs_status will no longer be a number of persons but a percentage.
}
\description{
Get distribution for ilostat number of persons only.
}
\details{
this function use the max of the corresponding grouping so it is 
		important to not filter any subset of the corresponding variable selected for the distribution
	at this level, ie. if you remove SEX_T, the distribution by sex will only have SEX_F or SEX_M / max(SEX_M, SEX_F) * 100,
	which is no longer a distribution.

	In addition, distribution is only applicable for indicators with Number of persons (usually in thousands), 
	So please do not distribute ratios, earnings, hours of works, CPI, GDP etc ... no warning will prevent for that
	if doubts use distribution from get_ilostat() instead of, warnings will help you.
}
\examples{
\dontrun{
 dat <- get_ilostat("EMP_TEMP_SEX_STE_GEO_NB_A", cache = FALSE)
 dat_dist <- distribution_ilostat(dat, "classif1")
 dat_plus_dist <- mutate(dat, dist = distribution_ilostat(dat,"classif1", .keep=TRUE))
 head(dat_dist)
 clean_ilostat_cache() 
}
}
\references{
See citation("Rilostat")
ilostat bulk download facility user guidelines 
\url{https://ilostat.ilo.org/data/bulk/}
}
\author{
David Bescond \email{bescond@ilo.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rilostat.R
\docType{package}
\name{Rilostat}
\alias{Rilostat-package}
\alias{Rilostat}
\title{R Tools for ilostat Open Data}
\description{
This package provides a user-friendly interface to access, download, and manage ILOSTAT open data.
}
\details{
On a regular basis new tutorials and examples are built and available through this function.

\tabular{ll}{
Package: \tab Rilostat\cr
Type: \tab Package\cr
Version: \tab See sessionInfo() or DESCRIPTION file\cr
Date: \tab 2020-2025\cr
License: \tab BSD_2_clause + file LICENSE\cr
LazyLoad: \tab yes\cr
}
}
\examples{
\dontrun{
# check which documentation have been recently added:
# help(Rilostat)

# https://ilostat.github.io/Rilostat/
}
}
\references{
See citation("Rilostat")

ilostat bulk download facility user guidelines
\url{https://ilostat.ilo.org/data/bulk/}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ilostat.github.io/Rilostat/}
  \item \url{https://github.com/ilostat/Rilostat}
  \item Report bugs at \url{https://github.com/ilostat/Rilostat/issues}
}

}
\author{
David Bescond \email{bescond@ilo.org}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_ilostat_cache.R
\name{clean_ilostat_cache}
\alias{clean_ilostat_cache}
\title{Clean ilostat Cache}
\usage{
clean_ilostat_cache(
  cache_dir = getOption("ilostat_cache_dir", file.path(tempdir(), "ilostat")),
  cache_update = getOption("ilostat_cache_update", FALSE),
  quiet = getOption("ilostat_quiet", TRUE)
)
}
\arguments{
\item{cache_dir}{A character, path to a cache directory. The directory has to exist.
The \code{NULL} (default) uses and creates
'ilostat' directory in the temporary directory from
\code{\link{tempdir}}. The directory can also be set with
\code{option} ilostat_cache_dir,}

\item{cache_update}{a logical whether to delete only out of date cache files. Useful when \code{cache_dir} 
is set as keep only update datasets. Can be set also with
  options(ilostat_update = TRUE). Default is \code{FALSE}.}

\item{quiet}{a logical, if \code{TRUE} , don't return message from processing, \code{FALSE} (default).
Can be set also with options(ilostat_quiet = TRUE),}
}
\description{
Deletes all cache files from the your ilostat cache directory.
             See \code{\link{get_ilostat}} for more on cache.
}
\examples{
\dontrun{
clean_ilostat_cache() 
}
}
\references{
See citation("Rilostat")
ilostat bulk download facility user guidelines 
\url{https://ilostat.ilo.org/data/bulk/}
}
\author{
David Bescond \email{bescond@ilo.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ilostat_toc.R
\name{get_ilostat_toc}
\alias{get_ilostat_toc}
\title{Read ilostat Table of Contents / workflow}
\usage{
get_ilostat_toc(
  segment = getOption("ilostat_segment", "indicator"),
  lang = getOption("ilostat_lang", "en"),
  search = getOption("ilostat_search", "none"),
  filters = getOption("ilostat_filter", "none"),
  fixed = getOption("ilostat_fixed", TRUE),
  cache_dir = getOption("ilostat_cache_dir", NULL),
  cache_update = getOption("ilostat_cache_update", TRUE),
  quiet = getOption("ilostat_quiet", TRUE)
)
}
\arguments{
\item{segment}{A character, way to get datasets by: \code{"indicator"} (default) or \code{"ref_area"}, 
Can be set also with options(ilostat_segment = "ref_area"),}

\item{lang}{a character, code for language. Available are \code{"en"} (default), 
\code{"fr"} and \code{"es"}. Can be set also with options(ilostat_lang = "fr"),}

\item{search}{a character vector, "none" (default), datasets with this pattern in
the description will be returned,
   characters vector will be use as AND, Character with "|" as OR, see example,
options(ilostat_time_format = "date"),}

\item{filters}{a list; \code{"none"} (default) to get a whole toc or a named list of
filters to get just part of the table. Names of list objects are
ilostat toc variable codes and values are vectors of observation codes.
filters detect on variables.}

\item{fixed}{a logical, if \code{TRUE} (default), pattern is a string to be matched as is,
Change to \code{FALSE} if more complex regex matching is needed.}

\item{cache_dir}{a path to a cache directory. The directory has to exist.
The \code{NULL} (default) uses and creates
'ilostat' directory in the temporary directory from
\code{\link{tempdir}}. The directory can also be set with
\code{option} ilostat_cache_dir,}

\item{cache_update}{a logical whether to update cache. Check cache update with last.update attribute store on the cache file name 
 and the one from the table of contents. Can be set also with
options(ilostat_cache_update = FALSE). Default is \code{TRUE},}

\item{quiet}{a logical, if \code{FALSE} , return message from processing, \code{TRUE} (default).
Can be set also with options(ilostat_quiet = TRUE),}
}
\value{
A tibble with ten columns depending of the segment: indicator or ref_area
		\itemize{
     \item{\code{id}} : The codename of dataset of theme, will be used by the get_ilostat and get_ilostat_raw functions,
     \item{\code{indicator or ref_area}} : The indicator or ref_area code of dataset,
     \item{\code{indicator.label or ref_area.label}} : The indicator or ref_area name of dataset,
     \item{\code{freq}}  : The frequency code of dataset,
     \item{\code{freq.label}} : Is freq name of dataset,
     \item{\code{size}} : Size of the csv.gz files,
     \item{\code{data.start}} : First time period of the dataset, 
     \item{\code{data.end}} : Last time period of the dataset,
     \item{\code{last.update}} : Last update of the dataset,
     \item{\code{...}} : Others relevant information
     }
}
\description{
Download one table of contents from ilostat \url{https://ilostat.ilo.org} via bulk download facility 
\url{https://ilostat.ilo.org/data/bulk/}.
}
\details{
The toc in English by indicator is downloaded from ilostat API \url{https://rplumber.ilo.org/__docs__/#get-/metadata/toc/indicator/}. 
The values in column "id" should be used to download a selected dataset.

The toc in English by ref_area is downloaded from ilostat API \url{https://rplumber.ilo.org/__docs__/#get-/metadata/toc/ref_area/}. 
The values in column "id" should be used to download a selected dataset.
}
\examples{
\dontrun{
## default segment by indicator, default lang English
 toc <- get_ilostat_toc()
 head(toc)
 toc <- get_ilostat_toc(segment = "ref_area", lang = "fr")
 head(toc)
##
## search on toc
 toc <- get_ilostat_toc(search = "education")
 head(toc)
 toc <- get_ilostat_toc(lang = "fr", search = "age")
 head(toc)
 toc <- get_ilostat_toc(segment = "ref_area", lang = "fr", search = "Albanie")
 toc
 toc <- get_ilostat_toc(segment = "ref_area", lang = "es", search = "Trimestral")
 head(toc)
##
## search multi on toc
 toc <- get_ilostat_toc(segment = "ref_area", lang = "fr", 
             search = "Albanie|France", fixed = FALSE)
 head(toc)
 toc <- get_ilostat_toc(search = "youth|adult", fixed = FALSE)
 head(toc)
##
}
}
\references{
See citation("Rilostat")
ilostat bulk download facility user guidelines \url{https://ilostat.ilo.org/data/bulk/}
}
\seealso{
\code{\link{get_ilostat}}.
}
\author{
David Bescond \email{bescond@ilo.org}
}
\keyword{database}
\keyword{utilities}
